# -*- coding: utf-8 -*-
import numpy as np
import matplotlib.pyplot as plt

import scipy.integrate
import time


# TODO vervollständigen Sie das Template.

#  Sie sollten eventuell aus `ode_solvers.py` und `runge_kutta.py`
#  importieren.

# -- Globale physikalische Parameter --------------------------------------------
# Bauteile
R = 100e3  # 100 KOhm
C = 10e-9  # 10 nF
L = 50e-3  # 50 mH

# Signal
Vo = 5  # 5 V
w = 50  # 50 Hz
Vin = lambda t: Vo * np.sin(2 * np.pi * w * t)

# Diodenmodell
Is = 1e-9  # 1 nA
Ut = 25e-3  # 25 mV
n = 1.0
Id = lambda t, yp: Is * (np.exp((Vin(t) - L * yp) / (n * Ut)) - 1.0)

# Parameter des ODE Lösers
n_periods = 1
n_nodes = 8000
n_steps = n_periods * n_nodes + 2 * n_nodes // 4 + 1
t_end = n_periods * 0.02 + 2 * 0.005


# TODO vervollständigen Sie das Template.



def plot_voltage(t, Vout, label):
    """Plottet die Spannung."""

    def plot():
        plt.figure()
        plt.plot(t, Vin(t), label=r"$V_{in}(t)$")
        plt.plot(t, Vout, label=r"$V_{out}(t)$")
        plt.xlabel(r"$t \; [s]$")
        plt.ylabel(r"$V \; [V]$")
        plt.ylim(-Vo - 1, Vo + 1)
        plt.grid(True)

    # Plot ohne Zoom
    plot()
    plt.legend(loc="lower right")
    plt.savefig("img/vosc_" + label + ".png")
    plt.savefig("img/vosc_" + label + ".pdf")

    # Plot mit Zoom
    plot()
    plt.xlim(0.016, 0.021)
    plt.legend(loc="upper right")
    plt.savefig("img/vosc_zoom_" + label + ".pdf")
    plt.savefig("img/vosc_zoom_" + label + ".png")


def plot_current(t, IR, IC, ID, IL, label):
    """Plottet den Stromfluss."""

    def plot():
        plt.figure()
        plt.plot(t, ID, label=r"$I_D(t)$")
        plt.plot(t, IR, label=r"$I_R(t)$")
        plt.plot(t, IC, label=r"$I_C(t)$")
        plt.plot(t, IL, label=r"$I_L(t)$")
        plt.grid(True)
        plt.xlabel(r"$t \; [s]$")
        plt.ylabel(r"$I \; [A]$")

    # Plot ohne Zoom
    plot()
    plt.legend(loc="upper right")
    plt.savefig("img/iosc_" + label + ".pdf")

    # Plot mit Zoom
    plot()
    plt.xlim(0.016, 0.021)
    plt.ylim(-0.0025, 0.0025)
    plt.legend(loc="upper right")
    plt.savefig("img/iosc_zoom_" + label + ".pdf")



# TODO vervollständigen Sie das Template.

