# coding: utf-8

import numpy as np
import matplotlib.pyplot as plt
from splitting_parameters import splitting_parameters

# konstanten fürs problem
omega2 = 1.0  # = g / l



# TODO vervollständigen Sie das Template.



def exact_solution(T, y0):
    n_steps = 2 ** 16
    t, y = approximate_solution(T, n_steps, y0)

    return t, y


def approximate_solution(T, N, y0):

    # TODO vervollständigen Sie das Template.

    return np.zeros(N+1), np.zeros((N+1, 2)) # Bitte ersetzen.


def plot_solution(T, N, y0):
    t_E, y_E = exact_solution(T, y0)
    t, y1 = approximate_solution(T, N, y0)
    plt.figure(figsize=(20, 10))
    plt.title("Auslenkung")
    plt.plot(t_E, y_E[:, 0], label="exact")
    plt.plot(t, y1[:, 0], label="BM63")
    plt.grid()
    plt.legend(loc="best")
    plt.savefig("img/auslenkung.pdf", bbox_inches="tight", dpi=300)

    plt.figure(figsize=(20, 10))
    plt.title("Phasenraum")
    plt.plot(y_E[:, 0], y_E[:, 1], label="exact")
    plt.plot(y1[:, 0], y1[:, 1], label="BM63")
    plt.grid()
    plt.legend(loc="best")
    plt.savefig("img/phasenraum.png", bbox_inches="tight", dpi=300)

    plt.show()


def convergence_rate(errors, resolutions):
    """ Berechnet die Konvergenzrate. """
    poly = np.polyfit(np.log(resolutions), np.log(errors), deg=1)
    return -poly[0]


def potential_energy(y):
    """Berechnet die potenzielle Energie von `y`.

    y : 2D-Array. Approximative Lösung der Pendelgleichung.
        Achse 0: Zeitschritte
        Achse 1: Winkel & Geschwindigkeit.
    """
    return omega2 * (1.0 - np.cos(y[:, 0]))


def kinetic_energy(y):
    """Berechnet die kinetische Energie von `y`.

    y : 2D-Array. Approximative Lösung der Pendelgleichung.
        Achse 0: Zeitschritte
        Achse 1: Winkel & Geschwindigkeit.
    """
    return 0.5 * y[:, 1] ** 2


def total_energy(y):
    return potential_energy(y) + kinetic_energy(y)


def plot_errors(T, y0):
    resolutions = 2 ** np.arange(9, 15)
    error = np.empty(resolutions.size)
    all_times = []
    energy_errors = []

    _, yE = exact_solution(T, y0)
    for k, N in enumerate(resolutions):
        t, y = approximate_solution(T, N, y0)

        error[k] = np.abs(y[-1, 0] - yE[-1, 0])
        energy_errors.append(np.abs(total_energy(y) - total_energy(yE[:1, :])))
        all_times.append(t)

    rate = convergence_rate(error, resolutions)
    fig = plt.figure()
    plt.loglog(resolutions, error, "-b")
    plt.savefig("img/pendulum_error.png", bbox_inches="tight", dpi=300)

    fig = plt.figure()
    for t, dE in zip(all_times, energy_errors):
        plt.semilogy(t, dE, "-")

    print("rate: " + str(rate) + "\n")
    plt.savefig("img/pendulum_dE.png", bbox_inches="tight", dpi=300)
    plt.show()


def ex_2():
    t_end = 400.0
    n_steps = 1000

    y0 = np.array([np.pi / 3, 0])

    plot_solution(t_end, n_steps, y0)
    plot_errors(t_end, y0)


if __name__ == "__main__":
    ex_2()
